@@ -33,12 +33,13 @@
 #include "idbm.h"
 #include "idbm_fields.h"
 #include "log.h"
-#include "util.h"
+#include "iscsi_util.h"
 #include "iscsi_settings.h"
 #include "transport.h"
 #include "iscsi_sysfs.h"
 #include "iface.h"
 #include "sysdeps.h"
+#include "fw_context.h"
 
 #define IDBM_HIDE	0    /* Hide parameter when print. */
 #define IDBM_SHOW	1    /* Show parameter when print. */
@@ -127,83 +128,6 @@ static struct idbm *db;
 	_n++; \
 } while(0)
 
-/*
- * from linux kernel
- */
-static char *strstrip(char *s)
-{
-	size_t size;
-	char *end;
-
-	size = strlen(s);
-	if (!size)
-		return s;
-
-	end = s + size - 1;
-	while (end >= s && isspace(*end))
-		end--;
-	*(end + 1) = '\0';
-
-	while (*s && isspace(*s))
-		s++;
-
-	return s;
-}
-
-static char *get_global_string_param(char *pathname, const char *key)
-{
-	FILE *f = NULL;
-	int len;
-	char *line, buffer[1024];
-	char *name = NULL;
-
-	if (!pathname) {
-		log_error("No pathname to load %s from", key);
-		return NULL;
-	}
-
-	len = strlen(key);
-	if ((f = fopen(pathname, "r"))) {
-		while ((line = fgets(buffer, sizeof (buffer), f))) {
-
-			line = strstrip(line);
-
-			if (strncmp(line, key, len) == 0) {
-				char *end = line + len;
-
-				/*
-				 * make sure ther is something after the
-				 * key.
-				 */
-				if (strlen(end))
-					name = strdup(line + len);
-			}
-		}
-		fclose(f);
-		if (name)
-			log_debug(5, "%s=%s", key, name);
-	} else
-		log_error("can't open %s configuration file %s", key, pathname);
-
-	return name;
-}
-
-char *get_iscsi_initiatorname(char *pathname)
-{
-	char *name;
-
-	name = get_global_string_param(pathname, "InitiatorName=");
-	if (!name)
-		log_error("An InitiatorName= is required, but was not "
-			  "found in %s", pathname);
-	return name;
-}
-
-char *get_iscsi_initiatoralias(char *pathname)
-{
-	return get_global_string_param(pathname, "InitiatorAlias=");
-}
-
 static void
 idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 {
@@ -212,9 +136,10 @@ idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 	__recinfo_int_o2(DISC_STARTUP, ri, r, startup, IDBM_SHOW,
 			"manual", "automatic", num, 1);
 	__recinfo_int_o6(DISC_TYPE, ri, r, type, IDBM_SHOW,
-			"sendtargets", "offload_send_targets", "slp", "isns",
+			"sendtargets", "isns", "offload_send_targets", "slp",
 			"static", "fw", num, 0);
-	if (r->type == DISCOVERY_TYPE_SENDTARGETS) {
+	switch (r->type) {
+	case DISCOVERY_TYPE_SENDTARGETS:
 		__recinfo_str(DISC_ST_ADDR, ri, r,
 			address, IDBM_SHOW, num, 0);
 		__recinfo_int(DISC_ST_PORT, ri, r,
@@ -238,7 +163,13 @@ idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 		__recinfo_int(DISC_ST_LOGIN_TMO, ri, r,
 			u.sendtargets.conn_timeo.login_timeout,
 			IDBM_SHOW, num, 1);
-		__recinfo_int(DISC_ST_REOPEN_MAX,ri, r,
+		__recinfo_int_o2(DISC_ST_USE_DISC_DAEMON, ri, r,
+			u.sendtargets.use_discoveryd,
+			IDBM_SHOW, "No", "Yes", num, 1);
+		__recinfo_int(DISC_ST_DISC_DAEMON_POLL_INVAL, ri, r,
+			u.sendtargets.discoveryd_poll_inval,
+			IDBM_SHOW, num, 1);
+		__recinfo_int(DISC_ST_REOPEN_MAX, ri, r,
 			u.sendtargets.reopen_max,
 			IDBM_SHOW, num, 1);
 		__recinfo_int(DISC_ST_AUTH_TMO, ri, r,
@@ -250,10 +181,25 @@ idbm_recinfo_discovery(discovery_rec_t *r, recinfo_t *ri)
 		__recinfo_int(DISC_ST_MAX_RECV_DLEN, ri, r,
 			      u.sendtargets.iscsi.MaxRecvDataSegmentLength,
 			      IDBM_SHOW, num, 1);
+		break;
+	case DISCOVERY_TYPE_ISNS:
+		__recinfo_str(DISC_ISNS_ADDR, ri, r,
+			address, IDBM_SHOW, num, 0);
+		__recinfo_int(DISC_ISNS_PORT, ri, r,
+			port, IDBM_SHOW, num, 0);
+		__recinfo_int_o2(DISC_ISNS_USE_DISC_DAEMON, ri, r,
+			u.isns.use_discoveryd,
+			IDBM_SHOW, "No", "Yes", num, 1);
+		__recinfo_int(DISC_ISNS_DISC_DAEMON_POLL_INVAL, ri, r,
+			u.isns.discoveryd_poll_inval,
+			IDBM_SHOW, num, 1);
+		break;
+	default:
+		break;
 	}
 }
 
-static void
+void
 idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 {
 	int num = 0, i;
@@ -288,9 +234,9 @@ idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 		      num, 0);
 	__recinfo_int(NODE_DISC_PORT, ri, r, disc_port, IDBM_SHOW,
 		      num, 0);
-	__recinfo_int_o6(NODE_DISC_TYPE, ri, r, disc_type,
-			 IDBM_SHOW, "send_targets", "offload_send_targets",
-			 "slp", "isns", "static", "fw", num, 0);
+	__recinfo_int_o6(NODE_DISC_TYPE, ri, r, disc_type, IDBM_SHOW,
+			 "send_targets", "isns", "offload_send_targets", "slp",
+			 "static", "fw", num, 0);
 	__recinfo_int(SESSION_INIT_CMDSN, ri, r,
 		      session.initial_cmdsn, IDBM_SHOW, num, 1);
 	__recinfo_int(SESSION_INIT_LOGIN_RETRY, ri, r,
@@ -324,6 +270,9 @@ idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 	__recinfo_int(SESSION_LU_RESET_TMO, ri, r,
 		      session.err_timeo.lu_reset_timeout,
 		      IDBM_SHOW, num, 1);
+	__recinfo_int(SESSION_TGT_RESET_TMO, ri, r,
+		      session.err_timeo.tgt_reset_timeout,
+		      IDBM_SHOW, num, 1);
 	__recinfo_int(SESSION_HOST_RESET_TMO, ri, r,
 		      session.err_timeo.host_reset_timeout,
 		      IDBM_SHOW, num, 1);
@@ -384,6 +333,10 @@ idbm_recinfo_node(node_rec_t *r, recinfo_t *ri)
 		__recinfo_int(key, ri, r, conn[i].timeo.noop_out_timeout,
 				IDBM_SHOW, num, 1);
 
+		sprintf(key, CONN_MAX_XMIT_DLEN, i);
+		__recinfo_int(key, ri, r,
+			conn[i].iscsi.MaxXmitDataSegmentLength, IDBM_SHOW,
+			num, 1);
 		sprintf(key, CONN_MAX_RECV_DLEN, i);
 		__recinfo_int(key, ri, r,
 			conn[i].iscsi.MaxRecvDataSegmentLength, IDBM_SHOW,
@@ -479,7 +432,10 @@ idbm_discovery_setup_defaults(discovery_rec_t *rec, discovery_type_e type)
 
 	rec->startup = ISCSI_STARTUP_MANUAL;
 	rec->type = type;
-	if (type == DISCOVERY_TYPE_SENDTARGETS) {
+	switch (type) {
+	case DISCOVERY_TYPE_SENDTARGETS:
+		rec->u.sendtargets.discoveryd_poll_inval = 30;
+		rec->u.sendtargets.use_discoveryd = 0;
 		rec->u.sendtargets.reopen_max = 5;
 		rec->u.sendtargets.auth.authmethod = 0;
 		rec->u.sendtargets.auth.password_length = 0;
@@ -489,7 +445,8 @@ idbm_discovery_setup_defaults(discovery_rec_t *rec, discovery_type_e type)
 		rec->u.sendtargets.conn_timeo.active_timeout=30;
 		rec->u.sendtargets.iscsi.MaxRecvDataSegmentLength =
 						DEF_INI_DISC_MAX_RECV_SEG_LEN;
-	} else if (type == DISCOVERY_TYPE_SLP) {
+		break;
+	case DISCOVERY_TYPE_SLP:
 		rec->u.slp.interfaces = NULL;
 		rec->u.slp.scopes = NULL;
 		rec->u.slp.poll_interval = 5 * 60;	/* 5 minutes */
@@ -497,8 +454,13 @@ idbm_discovery_setup_defaults(discovery_rec_t *rec, discovery_type_e type)
 		rec->u.slp.auth.password_length = 0;
 		rec->u.slp.auth.password_in_length = 0;
 		rec->u.slp.auth.password_in_length = 0;
-	} else if (type == DISCOVERY_TYPE_ISNS) {
-		/* to be implemented */
+		break;
+	case DISCOVERY_TYPE_ISNS:
+		rec->u.isns.use_discoveryd = 0;
+		rec->u.isns.discoveryd_poll_inval = -1;
+		break;
+	default:
+		break;
 	}
 }
 
@@ -545,6 +507,9 @@ setup_passwd_len:
 					    "unknown value format '%s' for "
 					    "parameter name '%s'",
 					    line_number, value, name);
+			} else {
+				log_error("unknown value format '%s' for "
+					  "parameter name '%s'", value, name);
 			}
 			break;
 		}
@@ -988,7 +953,8 @@ static int print_discovered_flat(void *data, node_rec_t *rec)
 	if (rec->disc_type != drec->type)
 		goto no_match;
 
-	if (drec->type == DISCOVERY_TYPE_SENDTARGETS) {
+	if (drec->type == DISCOVERY_TYPE_SENDTARGETS ||
+	    drec->type == DISCOVERY_TYPE_ISNS) {
 		if (rec->disc_port != drec->port ||
 		    strcmp(rec->disc_address, drec->address))
 			goto no_match;
@@ -1013,7 +979,7 @@ static int print_discovered_tree(void *data, node_rec_t *rec)
 	if (rec->disc_type != drec->type)
 		goto no_match;
 
-	if (drec->type == DISCOVERY_TYPE_SENDTARGETS) {
+	if (strlen(drec->address)) {
 		if (rec->disc_port != drec->port ||
 		    strcmp(rec->disc_address, drec->address))
 			goto no_match;
@@ -1025,7 +991,7 @@ no_match:
 	return -1;
 }
 
-int idbm_print_discovered(struct discovery_rec *drec, int info_level)
+static int idbm_print_discovered(discovery_rec_t *drec, int info_level)
 {
 	int num_found = 0;
 
@@ -1045,21 +1011,18 @@ int idbm_print_discovered(struct discovery_rec *drec, int info_level)
 	return num_found;
 }
 
-static int idbm_print_all_st(int info_level)
+static int idbm_for_each_drec(int type, char *config_root, void *data,
+			      idbm_drec_op_fn *fn)
 {
 	DIR *entity_dirfd;
 	struct dirent *entity_dent;
 	int found = 0;
-	char *disc_dir;
+	discovery_rec_t drec;
 	char *tmp_port;
 
-	disc_dir = malloc(PATH_MAX);
-	if (!disc_dir)
-		return 0;
-
-	entity_dirfd = opendir(ST_CONFIG_DIR);
+	entity_dirfd = opendir(config_root);
 	if (!entity_dirfd)
-		goto free_disc;
+		return found;
 
 	while ((entity_dent = readdir(entity_dirfd))) {
 		if (!strcmp(entity_dent->d_name, ".") ||
@@ -1071,44 +1034,97 @@ static int idbm_print_all_st(int info_level)
 		tmp_port = strchr(entity_dent->d_name, ',');
 		if (!tmp_port)
 			continue;
+		/*
+		 * pre 872 tools dumped the target portal symlinks in the isns
+		 * dir instead of the server. If we find one of those links
+		 * (by checking if there is a valid port) we skip it.
+		 */
+		if (strchr(tmp_port, ':') || strchr(tmp_port, '.'))
+			continue;
 		*tmp_port++ = '\0';
 
-		if (info_level >= 1) {
-			struct discovery_rec drec;
-
-			printf("DiscoveryAddress: %s,%s\n",
-				entity_dent->d_name, tmp_port);
-
-			memset(&drec, 0, sizeof(struct discovery_rec));
-			strlcpy(drec.address, entity_dent->d_name,
-				sizeof(drec.address));
-			drec.port = atoi(tmp_port);
-			drec.type = DISCOVERY_TYPE_SENDTARGETS;
-
-			found += idbm_print_discovered(&drec, info_level);
-		} else {
-			printf("%s:%s via sendtargets\n", entity_dent->d_name,
-			       tmp_port);
-			found++;
+		memset(&drec, 0, sizeof(drec));
+		if (idbm_discovery_read(&drec, type, entity_dent->d_name,
+					atoi(tmp_port))) {
+			log_error("Could not read discovery record for "
+				  "%s:%s.", entity_dent->d_name, tmp_port);
+			continue;
 		}
+
+		if (!fn(data, &drec))
+			found++;
 	}
 	closedir(entity_dirfd);
-free_disc:
-	free(disc_dir);
 	return found;
 }
 
+int idbm_for_each_st_drec(void *data, idbm_drec_op_fn *fn)
+{
+	return idbm_for_each_drec(DISCOVERY_TYPE_SENDTARGETS, ST_CONFIG_DIR,
+				  data, fn);
+}
+
+int idbm_for_each_isns_drec(void *data, idbm_drec_op_fn *fn)
+{
+	return idbm_for_each_drec(DISCOVERY_TYPE_ISNS, ISNS_CONFIG_DIR,
+				  data, fn);
+}
+
+static int __idbm_print_all_by_drec(void *data, struct discovery_rec *drec)
+{
+	int info_level = *(int *)data;
+	int rc;
+
+	if (info_level >= 1) {
+		printf("DiscoveryAddress: %s,%d\n",
+		       drec->address, drec->port);
+		rc = idbm_print_discovered(drec, info_level);
+		if (rc)
+			return 0;
+		else
+			return ENODEV;
+	} else {
+		printf("%s:%d via %s\n", drec->address, drec->port,
+		       drec->type == DISCOVERY_TYPE_ISNS ?
+		       "isns" : "sendtargets");
+		return 0;
+	}
+}
+
+static int idbm_print_all_st(int info_level)
+{
+	int rc;
+
+	rc = idbm_for_each_st_drec(&info_level, __idbm_print_all_by_drec);
+	if (rc < 0)
+		return 0;
+	return rc;
+}
+
+static int idbm_print_all_isns(int info_level)
+{
+	int rc;
+
+	rc = idbm_for_each_isns_drec(&info_level, __idbm_print_all_by_drec);
+	if (rc < 0)
+		return 0;
+	return rc;
+}
+
 int idbm_print_all_discovery(int info_level)
 {
 	discovery_rec_t *drec;
 	int found = 0, tmp;
 
-	if (info_level < 1)
-		return idbm_print_all_st(info_level);
+	if (info_level < 1) {
+		found = idbm_print_all_st(info_level);
+		found += idbm_print_all_isns(info_level);
+		return found;
+	}
 
 	drec = calloc(1, sizeof(*drec));
 	if (!drec)
-		return ENOMEM;
+		return 0;
 
 	tmp = 0;
 	printf("SENDTARGETS:\n");
@@ -1119,10 +1135,17 @@ int idbm_print_all_discovery(int info_level)
 	tmp = 0;
 
 	printf("iSNS:\n");
-	drec->type = DISCOVERY_TYPE_ISNS;
-	tmp = idbm_print_discovered(drec, info_level);
-	if (!tmp)
-		printf("No targets found.\n");
+	tmp = idbm_print_all_isns(info_level);
+	if (!tmp) {
+		/*
+		 * pre 872 tools did not store the server ip,port so
+		 * we drop down here, to just look for target portals.
+		 */
+		drec->type = DISCOVERY_TYPE_ISNS;
+		tmp = idbm_print_discovered(drec, info_level);
+		if (!tmp)
+			printf("No targets found.\n");
+	}
 	found += tmp;
 	tmp = 0;
 
@@ -1149,7 +1172,7 @@ int idbm_print_all_discovery(int info_level)
  * This iterates over the ifaces in use in the nodes dir.
  * It does not iterate over the ifaces setup in /etc/iscsi/ifaces.
  */
-static int idbm_for_each_iface(int *found, void *data,
+int idbm_for_each_iface(int *found, void *data,
 				idbm_iface_op_fn *fn,
 				char *targetname, int tpgt, char *ip, int port)
 {
@@ -1193,6 +1216,7 @@ read_iface:
 
 	iface_dirfd = opendir(portal);
 	if (!iface_dirfd) {
+		log_error("iface iter could not read dir %s.", portal);
 		rc = errno;
 		goto free_portal;
 	}
@@ -1334,14 +1358,26 @@ int idbm_for_each_rec(int *found, void *data, idbm_iface_op_fn *fn)
 	return idbm_for_each_node(found, &op_data, node_fn);
 }
 
+static struct {
+	char *config_root;
+	char *config_name;
+} disc_type_to_config_vals[] = {
+	{ ST_CONFIG_DIR, ST_CONFIG_NAME },
+	{ ISNS_CONFIG_DIR, ISNS_CONFIG_NAME },
+};
+
 int
-idbm_discovery_read(discovery_rec_t *out_rec, char *addr, int port)
+idbm_discovery_read(discovery_rec_t *out_rec, int drec_type,
+		    char *addr, int port)
 {
 	recinfo_t *info;
 	char *portal;
 	int rc = 0;
 	FILE *f;
 
+	if (drec_type > 1)
+		return EINVAL;
+
 	memset(out_rec, 0, sizeof(discovery_rec_t));
 
 	info = idbm_recinfo_alloc(MAX_KEYS);
@@ -1354,7 +1390,8 @@ idbm_discovery_read(discovery_rec_t *out_rec, char *addr, int port)
 		goto free_info;
 	}
 
-	snprintf(portal, PATH_MAX, "%s/%s,%d", ST_CONFIG_DIR,
+	snprintf(portal, PATH_MAX, "%s/%s,%d",
+		 disc_type_to_config_vals[drec_type].config_root,
 		 addr, port);
 	log_debug(5, "Looking for config file %s\n", portal);
 
@@ -1362,14 +1399,15 @@ idbm_discovery_read(discovery_rec_t *out_rec, char *addr, int port)
 	if (rc)
 		goto free_info;
 
-	f = idbm_open_rec_r(portal, ST_CONFIG_NAME);
+	f = idbm_open_rec_r(portal,
+			    disc_type_to_config_vals[drec_type].config_name);
 	if (!f) {
 		log_debug(1, "Could not open %s err %d\n", portal, errno);
 		rc = errno;
 		goto unlock;
 	}
 
-	idbm_discovery_setup_defaults(out_rec, DISCOVERY_TYPE_SENDTARGETS);
+	idbm_discovery_setup_defaults(out_rec, drec_type);
 	idbm_recinfo_discovery(out_rec, info);
 	idbm_recinfo_config(info, f);
 	fclose(f);
@@ -1542,6 +1580,9 @@ idbm_discovery_write(discovery_rec_t *rec)
 	char *portal;
 	int rc = 0;
 
+	if (rec->type > 1)
+		return EINVAL;
+
 	portal = malloc(PATH_MAX);
 	if (!portal) {
 		log_error("Could not alloc portal\n");
@@ -1552,7 +1593,8 @@ idbm_discovery_write(discovery_rec_t *rec)
 	if (rc)
 		goto free_portal;
 
-	snprintf(portal, PATH_MAX, "%s", ST_CONFIG_DIR);
+	snprintf(portal, PATH_MAX, "%s",
+		 disc_type_to_config_vals[rec->type].config_root);
 	if (access(portal, F_OK) != 0) {
 		if (mkdir(portal, 0660) != 0) {
 			log_error("Could not make %s\n", portal);
@@ -1561,10 +1603,12 @@ idbm_discovery_write(discovery_rec_t *rec)
 		}
 	}
 
-	snprintf(portal, PATH_MAX, "%s/%s,%d", ST_CONFIG_DIR,
+	snprintf(portal, PATH_MAX, "%s/%s,%d",
+		 disc_type_to_config_vals[rec->type].config_root,
 		 rec->address, rec->port);
 
-	f = idbm_open_rec_w(portal, ST_CONFIG_NAME);
+	f = idbm_open_rec_w(portal,
+			    disc_type_to_config_vals[rec->type].config_name);
 	if (!f) {
 		log_error("Could not open %s err %d\n", portal, errno);
 		rc = errno;
@@ -1580,26 +1624,23 @@ free_portal:
 	return rc;
 }
 
-int
-idbm_add_discovery(discovery_rec_t *newrec, int overwrite)
+int idbm_add_discovery(discovery_rec_t *newrec)
 {
 	discovery_rec_t rec;
-	int rc;
 
-	if (!idbm_discovery_read(&rec, newrec->address,
+	if (!idbm_discovery_read(&rec, newrec->type, newrec->address,
 				newrec->port)) {
-		if (!overwrite)
-			return 0;
-		log_debug(7, "overwriting existing record");
+		log_debug(7, "disc rec already exists");
+		/* fall through */
 	} else
 		log_debug(7, "adding new DB record");
 
-	rc = idbm_discovery_write(newrec);
-	return rc;
+	return idbm_discovery_write(newrec);
 }
 
 static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 {
+	struct stat statb;
 	int rc = 0;
 
 	switch (rec->disc_type) {
@@ -1648,10 +1689,46 @@ static int setup_disc_to_node_link(char *disc_portal, node_rec_t *rec)
 			}
 		}
 
+		/*
+		 * Older tools lumped all portals together in the
+		 * isns config dir. In 2.0-872, the isns dir added
+		 * a isns server (ddress and port) dir like sendtargets.
+		 *
+		 * If we found a older style link we return that so it
+		 * can be removed. If this function is called for
+		 * addition of a rec then the older link should have been
+		 * removed and we break down below.
+		 */
 		snprintf(disc_portal, PATH_MAX, "%s/%s,%s,%d,%d,%s",
 			 ISNS_CONFIG_DIR,
 			 rec->name, rec->conn[0].address,
 			 rec->conn[0].port, rec->tpgt, rec->iface.name);
+		if (!stat(disc_portal, &statb)) {
+			log_debug(7, "using old style isns dir %s.",
+				  disc_portal);
+			break;
+		}
+
+		snprintf(disc_portal, PATH_MAX, "%s/%s,%d",
+			 ISNS_CONFIG_DIR, rec->disc_address, rec->disc_port);
+		if (!stat(disc_portal, &statb) && S_ISDIR(statb.st_mode)) {
+			/*
+			 * if there is a dir for this isns server then
+			 * assume we are using the new style links
+			 */
+			snprintf(disc_portal, PATH_MAX,
+				 "%s/%s,%d/%s,%s,%d,%d,%s",
+				 ISNS_CONFIG_DIR, rec->disc_address,
+				 rec->disc_port, rec->name,
+				 rec->conn[0].address, rec->conn[0].port,
+				 rec->tpgt, rec->iface.name);
+			break;
+		}
+
+		/* adding a older link */
+		snprintf(disc_portal, PATH_MAX, "%s/%s,%s,%d,%d,%s",
+			 ISNS_CONFIG_DIR, rec->name, rec->conn[0].address,
+			 rec->conn[0].port, rec->tpgt, rec->iface.name);
 		break;
 	case DISCOVERY_TYPE_SLP:
 	default:
@@ -1730,15 +1807,14 @@ free_portal:
 }
 
 static int idbm_bind_iface_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
-				    struct discovery_rec *drec,
-				    struct iface_rec *iface,
+				    void *data, struct iface_rec *iface,
 				    struct list_head *bound_recs)
 {
 	struct node_rec *rec, *tmp;
 	struct list_head new_recs;
 
 	INIT_LIST_HEAD(&new_recs);
-	if (disc_node_fn(drec, iface, &new_recs))
+	if (disc_node_fn(data, iface, &new_recs))
 		return ENODEV;
 
 	list_for_each_entry_safe(rec, tmp, &new_recs, list) {
@@ -1750,21 +1826,21 @@ static int idbm_bind_iface_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
 }
 
 int idbm_bind_ifaces_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
-			      struct discovery_rec *drec,
-			      struct list_head *ifaces,
+			      void *data, struct list_head *ifaces,
 			      struct list_head *bound_recs)
 {
-	struct iface_rec *iface, *tmp;
+	struct list_head def_ifaces;
+	struct node_rec *rec, *tmp_rec;
+	struct iface_rec *iface, *tmp_iface;
 	struct iscsi_transport *t;
 	int rc = 0, found = 0;
 
-	if (!ifaces || list_empty(ifaces)) {
-		struct list_head def_ifaces;
+	INIT_LIST_HEAD(&def_ifaces);
 
-		INIT_LIST_HEAD(&def_ifaces);
+	if (!ifaces || list_empty(ifaces)) {
 		iface_link_ifaces(&def_ifaces);
 
-		list_for_each_entry_safe(iface, tmp, &def_ifaces, list) {
+		list_for_each_entry_safe(iface, tmp_iface, &def_ifaces, list) {
 			list_del(&iface->list);
 			t = iscsi_sysfs_get_transport_by_name(iface->transport_name);
 			/*
@@ -1777,11 +1853,11 @@ int idbm_bind_ifaces_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
 				continue;
 			}
 
-			rc = idbm_bind_iface_to_nodes(disc_node_fn, drec, iface,
+			rc = idbm_bind_iface_to_nodes(disc_node_fn, data, iface,
 						      bound_recs);
 			free(iface);
 			if (rc)
-				return rc;
+				goto fail;
 			found = 1;
 		}
 
@@ -1789,10 +1865,9 @@ int idbm_bind_ifaces_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
 		if (!found) {
 			struct iface_rec def_iface;
 
-			//log_error("no ifaces using default\n");
 			memset(&def_iface, 0, sizeof(struct iface_rec));
 			iface_setup_defaults(&def_iface);
-			return idbm_bind_iface_to_nodes(disc_node_fn, drec,
+			return idbm_bind_iface_to_nodes(disc_node_fn, data,
 							&def_iface, bound_recs);
 		}
 	} else {
@@ -1806,72 +1881,25 @@ int idbm_bind_ifaces_to_nodes(idbm_disc_nodes_fn *disc_node_fn,
 				continue;
 			}
 
-			rc = idbm_bind_iface_to_nodes(disc_node_fn, drec, iface,
+			rc = idbm_bind_iface_to_nodes(disc_node_fn, data, iface,
 						      bound_recs);
 			if (rc)
-				return rc;
+				goto fail;
 		}
 	}
 	return 0;
-}
 
-/*
- * remove this when isns is converted
- */
-int idbm_add_nodes(node_rec_t *newrec, discovery_rec_t *drec,
-		   struct list_head *ifaces, int update)
-{
-	struct iface_rec *iface, *tmp;
-	struct iscsi_transport *t;
-	int rc = 0, found = 0;
-
-	if (!ifaces || list_empty(ifaces)) {
-		struct list_head def_ifaces;
-
-		INIT_LIST_HEAD(&def_ifaces);
-		iface_link_ifaces(&def_ifaces);
-
-		list_for_each_entry_safe(iface, tmp, &def_ifaces, list) {
-			list_del(&iface->list);
-			t = iscsi_sysfs_get_transport_by_name(iface->transport_name);
-			/* only auto bind to software iscsi */
-			if (!t || strcmp(t->name, DEFAULT_TRANSPORT) ||
-			     !strcmp(iface->name, DEFAULT_IFACENAME)) {
-				free(iface);
-				continue;
-			}
-
-			iface_copy(&newrec->iface, iface);
-			rc = idbm_add_node(newrec, drec, update);
-			free(iface);
-			if (rc)
-				return rc;
-			found = 1;
-		}
-
-		/* create default iface with old/default behavior */
-		if (!found) {
-			iface_setup_defaults(&newrec->iface);
-			return idbm_add_node(newrec, drec, update);
-		}
-	} else {
-		list_for_each_entry(iface, ifaces, list) {
-			if (strcmp(iface->name, DEFAULT_IFACENAME) &&
-			    !iface_is_valid(iface)) {
-				log_error("iface %s is not valid. Will not "
-					  "bind node to it. Iface settings "
-					  iface_fmt, iface->name,
-					  iface_str(iface));
-				continue;
-			}
-
-			iface_copy(&newrec->iface, iface);
-			rc = idbm_add_node(newrec, drec, update);
-			if (rc)
-				return rc;
-		}
+fail:	
+	list_for_each_entry_safe(iface, tmp_iface, &def_ifaces, list) {
+		list_del(&iface->list);
+		free(iface);
 	}
-	return 0;
+
+	list_for_each_entry_safe(rec, tmp_rec, bound_recs, list) {
+		list_del(&rec->list);
+		free(rec);
+	}
+	return rc;
 }
 
 static void idbm_rm_disc_node_links(char *disc_dir)
@@ -1934,7 +1962,8 @@ int idbm_delete_discovery(discovery_rec_t *drec)
 	if (!portal)
 		return ENOMEM;
 
-	snprintf(portal, PATH_MAX, "%s/%s,%d", ST_CONFIG_DIR,
+	snprintf(portal, PATH_MAX, "%s/%s,%d",
+		 disc_type_to_config_vals[drec->type].config_root,
 		 drec->address, drec->port);
 	log_debug(5, "Removing config file %s\n", portal);
 
@@ -1946,21 +1975,25 @@ int idbm_delete_discovery(discovery_rec_t *drec)
 
 	if (S_ISDIR(statb.st_mode)) {
 		strlcat(portal, "/", PATH_MAX);
-		strlcat(portal, ST_CONFIG_NAME, PATH_MAX);
+		strlcat(portal,
+			disc_type_to_config_vals[drec->type].config_name,
+			PATH_MAX);
 	}
 
 	if (unlink(portal))
 		log_debug(5, "Could not remove %s err %d\n", portal, errno);
 
 	memset(portal, 0, PATH_MAX);
-	snprintf(portal, PATH_MAX, "%s/%s,%d", ST_CONFIG_DIR,
+	snprintf(portal, PATH_MAX, "%s/%s,%d",
+		 disc_type_to_config_vals[drec->type].config_root,
 		 drec->address, drec->port);
 	idbm_rm_disc_node_links(portal);
 
 	/* rm portal dir */
 	if (S_ISDIR(statb.st_mode)) {
 		memset(portal, 0, PATH_MAX);
-		snprintf(portal, PATH_MAX, "%s/%s,%d", ST_CONFIG_DIR,
+		snprintf(portal, PATH_MAX, "%s/%s,%d",
+			 disc_type_to_config_vals[drec->type].config_root,
 			 drec->address, drec->port);
 		rmdir(portal);
 	}
@@ -1998,8 +2031,8 @@ static int idbm_remove_disc_to_node_link(node_rec_t *rec,
 		goto done;
 	}
 
-	log_debug(7, "found drec %s %d\n", tmprec->disc_address,
-		 tmprec->disc_port); 
+	log_debug(7, "found drec %s %d\n",
+		  tmprec->disc_address, tmprec->disc_port);
 	/* rm link from discovery source to node */
 	memset(portal, 0, PATH_MAX);
 	rc = setup_disc_to_node_link(portal, tmprec);
@@ -2122,6 +2155,14 @@ idbm_sendtargets_defaults(struct iscsi_sendtargets_config *cfg)
 	       sizeof(struct iscsi_sendtargets_config));
 }
 
+void
+idbm_isns_defaults(struct iscsi_isns_config *cfg)
+{
+	idbm_sync_config();
+	memcpy(cfg, &db->drec_isns.u.isns,
+	       sizeof(struct iscsi_isns_config));
+}
+
 void
 idbm_slp_defaults(struct iscsi_slp_config *cfg)
 {
@@ -2158,6 +2199,35 @@ free_info:
 	return rc;
 }
 
+int idbm_discovery_set_param(void *data, discovery_rec_t *rec)
+{
+	struct db_set_param *param = data;
+	recinfo_t *info;
+	int rc = 0;
+
+	info = idbm_recinfo_alloc(MAX_KEYS);
+	if (!info)
+		return ENOMEM;
+
+	idbm_recinfo_discovery((discovery_rec_t *)rec, info);
+
+	rc = idbm_verify_param(info, param->name);
+	if (rc)
+		goto free_info;
+
+	rc = idbm_rec_update_param(info, param->name, param->value, 0);
+	if (rc)
+		goto free_info;
+
+	rc = idbm_discovery_write((discovery_rec_t *)rec);
+	if (rc)
+		goto free_info;
+
+free_info:
+	free(info);
+	return rc;
+}
+
 int idbm_init(idbm_get_config_file_fn *fn)
 {
 	/* make sure root db dir is there */
@@ -2184,3 +2254,159 @@ void idbm_terminate(void)
 	if (db)
 		free(db);
 }
+
+/**
+ * idbm_create_rec - allocate and setup a node record
+ * @targetname: target name
+ * @tgpt: target portal group
+ * @ip: ip address of portal
+ * @port: port of portal
+ * @iface: iscsi iface info
+ * @verbose: flag indicating whether to log ifaces setup errors
+ *
+ * The iface only needs to have the name set. This function will
+ * read in the other values.
+ */
+struct node_rec *idbm_create_rec(char *targetname, int tpgt, char *ip,
+				 int port, struct iface_rec *iface,
+				 int verbose)
+{
+	struct node_rec *rec;
+
+	rec = calloc(1, sizeof(*rec));
+	if (!rec) {
+		log_error("Could not not allocate memory to create node "
+			  "record.");
+		return NULL;
+	}
+
+	idbm_node_setup_defaults(rec);
+	if (targetname)
+		strlcpy(rec->name, targetname, TARGET_NAME_MAXLEN);
+	rec->tpgt = tpgt;
+	rec->conn[0].port = port;
+	if (ip)
+		strlcpy(rec->conn[0].address, ip, NI_MAXHOST);
+	memset(&rec->iface, 0, sizeof(struct iface_rec));
+	if (iface) {
+		iface_copy(&rec->iface, iface);
+		if (strlen(iface->name)) {
+			if (iface_conf_read(&rec->iface)) {
+				if (verbose)
+					log_error("Could not read iface info "
+						  "for %s.", iface->name);
+				goto free_rec;
+			}
+		}
+	}
+	return rec;
+free_rec:
+	free(rec);
+	return NULL;
+}
+
+struct node_rec *idbm_create_rec_from_boot_context(struct boot_context *context)
+{
+	struct node_rec *rec;
+
+	/* tpgt hard coded to 1 ??? */
+	rec = idbm_create_rec(context->targetname, 1,
+			      context->target_ipaddr, context->target_port,
+			      NULL, 1);
+	if (!rec) {
+		log_error("Could not setup rec for fw discovery login.");
+		return NULL;
+	}
+
+	iface_setup_defaults(&rec->iface);
+	strlcpy(rec->session.auth.username, context->chap_name,
+		sizeof(context->chap_name));
+	strlcpy((char *)rec->session.auth.password, context->chap_password,
+		sizeof(context->chap_password));
+	strlcpy(rec->session.auth.username_in, context->chap_name_in,
+		sizeof(context->chap_name_in));
+	strlcpy((char *)rec->session.auth.password_in,
+		context->chap_password_in,
+		sizeof(context->chap_password_in));
+	rec->session.auth.password_length =
+				strlen((char *)context->chap_password);
+	rec->session.auth.password_in_length =
+				strlen((char *)context->chap_password_in);
+
+	iface_setup_from_boot_context(&rec->iface, context);
+
+	return rec;
+}
+
+void idbm_node_setup_defaults(node_rec_t *rec)
+{
+	int i;
+
+	memset(rec, 0, sizeof(node_rec_t));
+
+	INIT_LIST_HEAD(&rec->list);
+
+	rec->tpgt = PORTAL_GROUP_TAG_UNKNOWN;
+	rec->disc_type = DISCOVERY_TYPE_STATIC;
+	rec->session.cmds_max = CMDS_MAX;
+	rec->session.xmit_thread_priority = XMIT_THREAD_PRIORITY;
+	rec->session.initial_cmdsn = 0;
+	rec->session.queue_depth = QUEUE_DEPTH;
+	rec->session.initial_login_retry_max = DEF_INITIAL_LOGIN_RETRIES_MAX;
+	rec->session.reopen_max = 32;
+	rec->session.auth.authmethod = 0;
+	rec->session.auth.password_length = 0;
+	rec->session.auth.password_in_length = 0;
+	rec->session.err_timeo.abort_timeout = DEF_ABORT_TIMEO;
+	rec->session.err_timeo.lu_reset_timeout = DEF_LU_RESET_TIMEO;
+	rec->session.err_timeo.tgt_reset_timeout = DEF_TGT_RESET_TIMEO;
+	rec->session.err_timeo.host_reset_timeout = DEF_HOST_RESET_TIMEO;
+	rec->session.timeo.replacement_timeout = DEF_REPLACEMENT_TIMEO;
+	rec->session.iscsi.InitialR2T = 0;
+	rec->session.iscsi.ImmediateData = 1;
+	rec->session.iscsi.FirstBurstLength = DEF_INI_FIRST_BURST_LEN;
+	rec->session.iscsi.MaxBurstLength = DEF_INI_MAX_BURST_LEN;
+	rec->session.iscsi.DefaultTime2Wait = ISCSI_DEF_TIME2WAIT;
+	rec->session.iscsi.DefaultTime2Retain = 0;
+	rec->session.iscsi.MaxConnections = 1;
+	rec->session.iscsi.MaxOutstandingR2T = 1;
+	rec->session.iscsi.ERL = 0;
+	rec->session.iscsi.FastAbort = 1;
+
+	for (i=0; i<ISCSI_CONN_MAX; i++) {
+		rec->conn[i].startup = ISCSI_STARTUP_MANUAL;
+		rec->conn[i].port = ISCSI_LISTEN_PORT;
+		rec->conn[i].tcp.window_size = TCP_WINDOW_SIZE;
+		rec->conn[i].tcp.type_of_service = 0;
+		rec->conn[i].timeo.login_timeout= DEF_LOGIN_TIMEO;
+		rec->conn[i].timeo.logout_timeout= DEF_LOGOUT_TIMEO;
+		rec->conn[i].timeo.auth_timeout = 45;
+
+		rec->conn[i].timeo.noop_out_interval = DEF_NOOP_OUT_INTERVAL;
+		rec->conn[i].timeo.noop_out_timeout = DEF_NOOP_OUT_TIMEO;
+
+		rec->conn[i].iscsi.MaxXmitDataSegmentLength = 0;
+		rec->conn[i].iscsi.MaxRecvDataSegmentLength =
+						DEF_INI_MAX_RECV_SEG_LEN;
+		rec->conn[i].iscsi.HeaderDigest = CONFIG_DIGEST_NEVER;
+		rec->conn[i].iscsi.DataDigest = CONFIG_DIGEST_NEVER;
+		rec->conn[i].iscsi.IFMarker = 0;
+		rec->conn[i].iscsi.OFMarker = 0;
+	}
+
+	iface_setup_defaults(&rec->iface);
+}
+
+struct node_rec *
+idbm_find_rec_in_list(struct list_head *rec_list, char *targetname, char *addr,
+		      int port, struct iface_rec *iface)
+{
+	struct node_rec *rec;
+
+	list_for_each_entry(rec, rec_list, list) {
+		if (__iscsi_match_session(rec, targetname, addr, port, iface))
+			return rec;
+	}
+
+	return NULL;
+}
