@@ -22,6 +22,7 @@
 
 #include <stdint.h>
 #include <net/if.h>
+#include <sys/time.h>
 
 #include "types.h"
 #include "iscsi_proto.h"
@@ -30,7 +31,6 @@
 #include "mgmt_ipc.h"
 #include "config.h"
 #include "actor.h"
-#include "queue.h"
 #include "list.h"
 
 #define ISCSI_CONFIG_ROOT	"/etc/iscsi/"
@@ -43,6 +43,23 @@
 #define LOCK_FILE		"/var/lock/iscsi/lock"
 #define LOCK_WRITE_FILE		"/var/lock/iscsi/lock.write"
 
+typedef enum iscsi_conn_state_e {
+	STATE_FREE,
+	STATE_XPT_WAIT,
+	STATE_IN_LOGIN,
+	STATE_LOGGED_IN,
+	STATE_IN_LOGOUT,
+	STATE_LOGOUT_REQUESTED,
+	STATE_CLEANUP_WAIT,
+} iscsi_conn_state_e;
+
+typedef enum iscsi_session_r_stage_e {
+	R_STAGE_NO_CHANGE,
+	R_STAGE_SESSION_CLEANUP,
+	R_STAGE_SESSION_REOPEN,
+	R_STAGE_SESSION_REDIRECT,
+} iscsi_session_r_stage_e;
+
 typedef enum conn_login_status_e {
 	CONN_LOGIN_SUCCESS		= 0,
 	CONN_LOGIN_FAILED		= 1,
@@ -59,25 +76,20 @@ enum iscsi_login_status {
 	LOGIN_VERSION_MISMATCH		= 3,
 	LOGIN_NEGOTIATION_FAILED	= 4,
 	LOGIN_AUTHENTICATION_FAILED	= 5,
-	LOGIN_WRONG_PORTAL_GROUP	= 6,
-	LOGIN_REDIRECTION_FAILED	= 7,
-	LOGIN_INVALID_PDU		= 8,
-	LOGIN_REDIRECT			= 9,
+	LOGIN_REDIRECTION_FAILED	= 6,
+	LOGIN_INVALID_PDU		= 7,
+	LOGIN_REDIRECT			= 8,
 };
 
 typedef enum iscsi_event_e {
-	EV_UNKNOWN			= 0,
-	EV_CONN_RECV_PDU		= 1,
-	EV_CONN_POLL			= 2,
-	EV_CONN_TIMER			= 3,
-	EV_CONN_ERROR			= 4,
+	EV_UNKNOWN,
+	EV_CONN_RECV_PDU,
+	EV_CONN_POLL,
+	EV_CONN_ERROR,
+	EV_CONN_LOGOUT_TIMER,
+	EV_CONN_STOP,
 } iscsi_event_e;
 
-typedef struct iscsi_event {
-	queue_item_t item;
-	char payload[EVENT_PAYLOAD_MAX];
-} iscsi_event_t;
-
 struct queue_task;
 
 typedef struct iscsi_login_context {
@@ -100,49 +112,26 @@ typedef struct iscsi_login_context {
 
 struct iscsi_session;
 struct iscsi_conn;
-
-typedef void (*send_pdu_begin_f) (uint64_t transport_handle, uint32_t sid,
-                                  uint32_t cid, int hdr_size, int data_size);
-typedef int (*send_pdu_end_f) (uint64_t transport_handle, uint32_t sid,
-			       uint32_t cid, int *retcode);
-typedef int (*recv_pdu_begin_f) (uint64_t transport_handle,
-				 uintptr_t recv_handle, uintptr_t *pdu_handle,
-				 int *pdu_size);
-typedef int (*recv_pdu_end_f) (uint64_t transport_handle, uintptr_t conn_handle,
-                             uintptr_t pdu_handle);
-typedef void (*send_pdu_timer_add_f)(struct iscsi_conn *conn, int timeout);
-typedef void (*send_pdu_timer_remove_f)(struct iscsi_conn *conn);
+struct iscsi_conn_context;
 
 /* daemon's connection structure */
 typedef struct iscsi_conn {
-	struct qelem item; /* must stay at the top */
 	uint32_t id;
-	uintptr_t recv_handle;
 	struct iscsi_session *session;
 	iscsi_login_context_t login_context;
+	struct iscsi_conn_context *recv_context;
 	struct queue_task *logout_qtask;
 	char data[ISCSI_DEF_MAX_RECV_SEG_LEN];
 	char host[NI_MAXHOST];	/* scratch */
 	iscsi_conn_state_e state;
-	actor_t connect_timer;
-	actor_t send_pdu_timer;
-	actor_t logout_timer;
+	int userspace_nop;
 
-	actor_t noop_out_timer;
-	actor_t noop_out_timeout_timer;
+	struct timeval initial_connect_time;
+	actor_t login_timer;
+	actor_t nop_out_timer;
 
-	int send_pdu_in_progress;
-
-	int kernel_io;
-	send_pdu_begin_f send_pdu_begin;
-	send_pdu_end_f send_pdu_end;
-	recv_pdu_begin_f recv_pdu_begin;
-	recv_pdu_end_f recv_pdu_end;
-	send_pdu_timer_add_f send_pdu_timer_add;
-	send_pdu_timer_remove_f send_pdu_timer_remove;
-
-#define RECVPOOL_MAX	32
-	void* recvpool[RECVPOOL_MAX];
+#define CONTEXT_POOL_MAX 32
+	struct iscsi_conn_context *context_pool[CONTEXT_POOL_MAX];
 
 	/* login state machine */
 	int current_stage;
@@ -154,7 +143,7 @@ typedef struct iscsi_conn {
 	int socket_fd;
 	/* address being used for normal session connection */
 	struct sockaddr_storage saddr;
-	/* address recieved during login */
+	/* address received during login */
 	struct sockaddr_storage failback_saddr;
 	int tcp_window_size;
 	int type_of_service;
@@ -163,14 +152,13 @@ typedef struct iscsi_conn {
          * transports (eg iser) which does these ops from the kernel.
          * In the case of TCP, it is just the transport_fd casted to u64. */
 	uint64_t transport_ep_handle;
+	int bind_ep;
 
 	/* timeouts */
 	int login_timeout;
 	int logout_timeout;
 	int auth_timeout;
 	int active_timeout;
-	int idle_timeout;
-	int ping_timeout;
 
 	int noop_out_interval;
 	int noop_out_timeout;
@@ -185,11 +173,22 @@ typedef struct iscsi_conn {
 	uint32_t max_xmit_dlength;	/* the value declared by the target */
 } iscsi_conn_t;
 
+struct iscsi_conn_context {
+	struct actor actor;
+	struct iscsi_conn *conn;
+	int allocated;
+	void *data;
+};
+
 typedef struct queue_task {
 	iscsi_conn_t *conn;
 	iscsiadm_req_t req;
 	iscsiadm_rsp_t rsp;
 	int mgmt_ipc_fd;
+	int allocated : 1;
+	/* Newer request types include a
+	 * variable-length payload */
+	void *payload;
 } queue_task_t;
 
 struct iscsi_transport_template;
@@ -200,7 +199,6 @@ typedef struct iscsi_session {
 	struct list_head list;
 	uint32_t id;
 	uint32_t hostno;
-	int refcount;
 	char netdev[IFNAMSIZ];
 	struct iscsi_transport *t;
 	node_rec_t nrec; /* copy of original Node record in database */
@@ -213,19 +211,17 @@ typedef struct iscsi_session {
 	int erl;
 	uint32_t imm_data_en;
 	uint32_t initial_r2t_en;
+	uint32_t fast_abort;
 	uint32_t first_burst;
 	uint32_t max_burst;
 	uint32_t pdu_inorder_en;
 	uint32_t dataseq_inorder_en;
-	uint32_t time2wait;
 	uint32_t def_time2wait;
 	uint32_t def_time2retain;
 	int type;
 	int portal_group_tag;
 	uint8_t isid[6];
 	uint16_t tsih;
-	int channel;
-	int target_id;
 	char target_name[TARGET_NAME_MAXLEN + 1];
 	char *target_alias;
 	char *initiator_name;
@@ -247,7 +243,7 @@ typedef struct iscsi_session {
 	int password_in_length;
 	iscsi_conn_t conn[ISCSI_CONN_MAX];
 	int ctrl_fd;
-	uint32_t param_mask;
+	uint64_t param_mask;
 
 	/* connection reopens during recovery */
 	int reopen_cnt;
@@ -255,41 +251,15 @@ typedef struct iscsi_session {
 	iscsi_session_r_stage_e r_stage;
 	uint32_t replacement_timeout;
 
+	int host_reset_timeout;
+	int tgt_reset_timeout;
+	int lu_reset_timeout;
+	int abort_timeout;
+
 	/* sync up fields */
 	queue_task_t *sync_qtask;
-
-	/* session's processing */
-	actor_t mainloop;
-	queue_t *queue;
-	queue_t *splice_queue;
-
 } iscsi_session_t;
 
-/*
- * TODO: replace session_info and host_info with
- * node_rec and iface_rec
- */
-struct session_info {
-	struct list_head list;
-	/* local info */
-	struct iface_rec iface;
-	int sid;
-
-	/* remote info */	
-	char targetname[TARGET_NAME_MAXLEN + 1];
-	int tpgt;
-	char address[NI_MAXHOST + 1];
-	int port;
-	char persistent_address[NI_MAXHOST + 1];
-	int persistent_port;
-};
-
-struct host_info {
-	char iname[TARGET_NAME_MAXLEN + 1];
-	struct iface_rec iface;
-	int host_no;
-};
-
 /* login.c */
 
 #define ISCSI_SESSION_TYPE_NORMAL 0
@@ -359,12 +329,14 @@ extern int iscsi_io_recv_pdu(iscsi_conn_t *conn, struct iscsi_hdr *hdr,
 
 /* initiator.c */
 extern int session_login_task(node_rec_t *rec, queue_task_t *qtask);
-extern int session_logout_task(iscsi_session_t *session, queue_task_t *qtask);
+extern int session_logout_task(int sid, queue_task_t *qtask);
 extern iscsi_session_t *session_find_by_sid(int sid);
-extern iscsi_session_t *session_find_by_rec(node_rec_t *rec);
-extern int session_is_running(node_rec_t *rec);
-extern void* recvpool_get(iscsi_conn_t *conn, int ev_size);
-extern void recvpool_put(iscsi_conn_t *conn, void *handle);
+extern struct iscsi_conn_context *iscsi_conn_context_get(iscsi_conn_t *conn,
+						   int ev_size);
+extern void iscsi_conn_context_put(struct iscsi_conn_context *conn_context);
+extern void iscsi_sched_conn_context(struct iscsi_conn_context *context,
+				     struct iscsi_conn *conn, unsigned long tmo,
+				     int event);
 extern mgmt_ipc_err_e iscsi_sync_session(node_rec_t *rec, queue_task_t
 					 *tsk, uint32_t sid);
 extern mgmt_ipc_err_e iscsi_host_send_targets(queue_task_t *qtask,
@@ -374,10 +346,4 @@ extern void iscsi_async_session_creation(uint32_t host_no, uint32_t sid);
 extern void iscsi_async_session_destruction(uint32_t host_no, uint32_t sid);
 extern void free_initiator(void);
 
-/* isns.c */
-extern int isns_init(void);
-extern void isns_handle(int);
-extern void isns_exit(void);
-extern int isns_dev_attr_query_task(queue_task_t *qtask);
-
 #endif /* INITIATOR_H */
