@@ -38,13 +38,15 @@
 #include "event_poll.h"
 #include "transport.h"
 #include "log.h"
-#include "util.h"
+#include "iscsi_util.h"
 #include "idbm.h"
 #include "version.h"
 #include "iscsi_sysfs.h"
 #include "iscsi_settings.h"
 #include "fw_context.h"
+#include "iface.h"
 #include "sysdeps.h"
+#include "iscsid_req.h"
 
 /* global config info */
 /* initiator needs initiator name/alias */
@@ -73,6 +75,7 @@ static struct option const long_options[] = {
 	{"password_in", required_argument, NULL, 'W'},
 	{"debug", required_argument, NULL, 'd'},
 	{"fwparam_connect", no_argument, NULL, 'b'},
+	{"fwparam_network", no_argument, NULL, 'N'},
 	{"fwparam_print", no_argument, NULL, 'f'},
 	{"help", no_argument, NULL, 'h'},
 	{"version", no_argument, NULL, 'v'},
@@ -96,10 +99,11 @@ Open-iSCSI initiator.\n\
   -u, --username=N         set username to N (optional)\n\
   -w, --password=N         set password to N (optional\n\
   -U, --username_in=N      set incoming username to N (optional)\n\
-  -W, --password_in=N      set incoming password to N (optional\n\
-  -d, --debug debuglevel   print debugging information \n\
-  -b, --fwparam_connect    create a session to the target\n\
-  -f, --fwparam_print      print the iBFT to STDOUT \n\
+  -W, --password_in=N      set incoming password to N (optional)\n\
+  -d, --debug=debuglevel   print debugging information \n\
+  -b, --fwparam_connect    create a session to the target using iBFT or OF\n\
+  -N, --fwparam_network    bring up the network as specified by iBFT or OF\n\
+  -f, --fwparam_print      print the iBFT or OF info to STDOUT \n\
   -h, --help               display this help and exit\n\
   -v, --version            display version and exit\n\
 ");
@@ -115,7 +119,7 @@ static int stop_event_loop(void)
 
 	memset(&req, 0, sizeof(req));
 	req.command = MGMT_IPC_IMMEDIATE_STOP;
-	rc = do_iscsid(&req, &rsp);
+	rc = iscsid_exec_req(&req, &rsp, 0);
 	if (rc) {
 		iscsid_handle_error(rc);
 		log_error("Could not stop event_loop\n");
@@ -124,7 +128,7 @@ static int stop_event_loop(void)
 }
 
 
-static int login_session(void)
+static int login_session(struct node_rec *rec)
 {
 	iscsiadm_req_t req;
 	iscsiadm_rsp_t rsp;
@@ -133,20 +137,20 @@ static int login_session(void)
 	 * For root boot we cannot change this so increase to account
 	 * for boot using static setup.
 	 */
-	config_rec.session.initial_login_retry_max = 30;
+	rec->session.initial_login_retry_max = 30;
 	/* we cannot answer so turn off */
-	config_rec.conn[0].timeo.noop_out_interval = 0;
-	config_rec.conn[0].timeo.noop_out_timeout = 0;
+	rec->conn[0].timeo.noop_out_interval = 0;
+	rec->conn[0].timeo.noop_out_timeout = 0;
 
-	printf("%s: Logging into %s %s:%d,%d\n", program_name, config_rec.name,
-		config_rec.conn[0].address, config_rec.conn[0].port,
-		config_rec.tpgt);
+	printf("%s: Logging into %s %s:%d,%d\n", program_name, rec->name,
+		rec->conn[0].address, rec->conn[0].port,
+		rec->tpgt);
 	memset(&req, 0, sizeof(req));
 	req.command = MGMT_IPC_SESSION_LOGIN;
-	memcpy(&req.u.session.rec, &config_rec, sizeof(node_rec_t));
+	memcpy(&req.u.session.rec, rec, sizeof(*rec));
 
 retry:
-	rc = do_iscsid(&req, &rsp);
+	rc = iscsid_exec_req(&req, &rsp, 0);
 	/*
 	 * handle race where iscsid proc is starting up while we are
 	 * trying to connect.
@@ -163,37 +167,28 @@ retry:
 static int setup_session(void)
 {
 	struct boot_context *context;
-	struct iscsi_auth_config *auth;
 	int rc = 0, rc2 = 0;
 
 	if (list_empty(&targets))
-		return login_session();
+		return login_session(&config_rec);
 
 	list_for_each_entry(context, &targets, list) {
-		idbm_node_setup_defaults(&config_rec);
+		struct node_rec *rec;
 
-		auth = &config_rec.session.auth;
-		strlcpy(config_rec.name, context->targetname,
-			sizeof(context->targetname));
-		strlcpy(config_rec.conn[0].address, context->target_ipaddr,
-			sizeof(context->target_ipaddr));
-		config_rec.conn[0].port = context->target_port;
-		/* this seems broken ??? */
-		config_rec.tpgt = 1;
-		strlcpy(auth->username, context->chap_name,
-			sizeof(context->chap_name));
-		strlcpy((char *)auth->password, context->chap_password,
-			sizeof(context->chap_password));
-		auth->password_length = strlen((char *)auth->password);
-		strlcpy(auth->username_in, context->chap_name_in,
-			sizeof(context->chap_name_in));
-		strlcpy((char *)auth->password_in, context->chap_password_in,
-			sizeof(context->chap_password_in));
-		auth->password_in_length = strlen((char *)auth->password_in);
+		rec = idbm_create_rec_from_boot_context(context);
+		if (!rec) {
+			log_error("Could not allocate memory. Could "
+				  "not start boot session to "
+				  "%s,%s,%d", context->targetname,
+				  context->target_ipaddr,
+				  context->target_port);
+			continue;
+		}
 
-		rc2 = login_session();
+		rc2 = login_session(rec);
 		if (rc2)
 			rc = rc2;
+		free(rec);
 	}
 	fw_free_targets(&targets);
 	return rc;
@@ -261,14 +256,13 @@ int main(int argc, char *argv[])
 	sigaction(SIGINT, &sa_new, &sa_old );
 
 	/* initialize logger */
-	log_daemon = 0;
-	log_init(program_name, DEFAULT_AREA_SIZE);
+	log_init(program_name, DEFAULT_AREA_SIZE, log_do_log_std, NULL);
 
 	sysfs_init();
 	if (iscsi_sysfs_check_class_version())
 		exit(1);
 
-	while ((ch = getopt_long(argc, argv, "i:t:g:a:p:d:u:w:U:W:bfvh",
+	while ((ch = getopt_long(argc, argv, "i:t:g:a:p:d:u:w:U:W:bNfvh",
 				 long_options, &longindex)) >= 0) {
 		switch (ch) {
 		case 'i':
@@ -332,6 +326,9 @@ int main(int argc, char *argv[])
 				exit(1);
 			}
 			break;
+		case 'N':
+			ret = fw_setup_nics();
+			exit(ret);
 		case 'f':
 			ret = fw_get_targets(&targets);
 			if (ret || list_empty(&targets)) {
@@ -424,7 +421,7 @@ int main(int argc, char *argv[])
 	 * Start Main Event Loop
 	 */
 	actor_init();
-	event_loop(ipc, control_fd, mgmt_ipc_fd, -1);
+	event_loop(ipc, control_fd, mgmt_ipc_fd);
 	ipc->ctldev_close();
 	mgmt_ipc_close(mgmt_ipc_fd);
 	free_initiator();
