@@ -12,13 +12,14 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <signal.h>
+#include <errno.h>
 #include <sys/shm.h>
 #include <sys/ipc.h>
 #include <sys/types.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include "util.h"
+#include "iscsi_util.h"
 #include "log.h"
 
 #define SEMKEY	0xA7L
@@ -31,10 +32,38 @@
 #endif
 
 char *log_name;
-int log_daemon = 1;
 int log_level = 0;
 
 static int log_stop_daemon = 0;
+static void (*log_func)(int prio, void *priv, const char *fmt, va_list ap);
+static void *log_func_priv;
+
+static void free_logarea (void)
+{
+	int shmid;
+
+	if (!la)
+		return;
+
+	if (la->semid != -1)
+		semctl(la->semid, 0, IPC_RMID, la->semarg);
+	if (la->buff) {
+		shmdt(la->buff);
+		shmctl(la->shmid_buff, IPC_RMID, NULL);
+		la->buff = NULL;
+		la->shmid_buff = -1;
+	}
+	if (la->start) {
+		shmdt(la->start);
+		shmctl(la->shmid_msg, IPC_RMID, NULL);
+		la->start = NULL;
+		la->shmid_msg = -1;
+	}
+	shmid = la->shmid;
+	shmdt(la);
+	shmctl(shmid, IPC_RMID, NULL);
+	la = NULL;
+}
 
 static int logarea_init (int size)
 {
@@ -43,25 +72,37 @@ static int logarea_init (int size)
 	logdbg(stderr,"enter logarea_init\n");
 
 	if ((shmid = shmget(IPC_PRIVATE, sizeof(struct logarea),
-			    0644 | IPC_CREAT | IPC_EXCL)) == -1)
+			    0644 | IPC_CREAT | IPC_EXCL)) == -1) {
+		syslog(LOG_ERR, "shmget logarea failed %d", errno);
 		return 1;
+	}
 
 	la = shmat(shmid, NULL, 0);
-	if (!la)
+	if (!la) {
+		syslog(LOG_ERR, "shmat logarea failed %d", errno);
+		shmctl(shmid, IPC_RMID, NULL);
 		return 1;
+	}
+	la->shmid = shmid;
+	la->start = NULL;
+	la->buff = NULL;
+	la->semid = -1;
 
 	if (size < MAX_MSG_SIZE)
 		size = DEFAULT_AREA_SIZE;
 
 	if ((shmid = shmget(IPC_PRIVATE, size,
 			    0644 | IPC_CREAT | IPC_EXCL)) == -1) {
-		shmdt(la);
+		syslog(LOG_ERR, "shmget msg failed %d", errno);
+		free_logarea();
 		return 1;
 	}
+	la->shmid_msg = shmid;
 
-	la->start = shmat(shmid, NULL, 0);
+	la->start = shmat(la->shmid_msg, NULL, 0);
 	if (!la->start) {
-		shmdt(la);
+		syslog(LOG_ERR, "shmat msg failed %d", errno);
+		free_logarea();
 		return 1;
 	}
 	memset(la->start, 0, size);
@@ -73,32 +114,31 @@ static int logarea_init (int size)
 
 	if ((shmid = shmget(IPC_PRIVATE, MAX_MSG_SIZE + sizeof(struct logmsg),
 			    0644 | IPC_CREAT | IPC_EXCL)) == -1) {
-		shmdt(la->start);
-		shmdt(la);
+		syslog(LOG_ERR, "shmget logmsg failed %d", errno);
+		free_logarea();
 		return 1;
 	}
 	la->buff = shmat(shmid, NULL, 0);
 	if (!la->buff) {
-		shmdt(la->start);
-		shmdt(la);
+		syslog(LOG_ERR, "shmat logmsgfailed %d", errno);
+		free_logarea();
 		return 1;
 	}
 
 	if ((la->semid = semget(SEMKEY, 1, 0600 | IPC_CREAT)) < 0) {
-		shmdt(la->buff);
-		shmdt(la->start);
-		shmdt(la);
+		syslog(LOG_ERR, "semget failed %d", errno);
+		free_logarea();
 		return 1;
 	}
 
 	la->semarg.val=1;
 	if (semctl(la->semid, 0, SETVAL, la->semarg) < 0) {
-		shmdt(la->buff);
-		shmdt(la->start);
-		shmdt(la);
+		syslog(LOG_ERR, "semctl failed %d", errno);
+		free_logarea();
 		return 1;
 	}
 
+	la->shmid_buff = shmid;
 	la->ops[0].sem_num = 0;
 	la->ops[0].sem_flg = 0;
 
@@ -106,15 +146,6 @@ static int logarea_init (int size)
 
 }
 
-static void free_logarea (void)
-{
-	semctl(la->semid, 0, IPC_RMID, la->semarg);
-	shmdt(la->buff);
-	shmdt(la->start);
-	shmdt(la);
-	return;
-}
-
 #if LOGDBG
 static void dump_logarea (void)
 {
@@ -196,7 +227,7 @@ int log_dequeue (void * buff)
 	int len;
 
 	if (la->empty)
-		return 1;
+		return 0;
 
 	len = strlen((char *)&src->str) * sizeof(char) +
 	      sizeof(struct logmsg) + 1;
@@ -215,7 +246,7 @@ int log_dequeue (void * buff)
 
 	memset((void *)src, 0, len);
 
-	return la->empty;
+	return len;
 }
 
 /*
@@ -228,22 +259,31 @@ static void log_syslog (void * buff)
 	syslog(msg->prio, "%s", (char *)&msg->str);
 }
 
-static void dolog(int prio, const char *fmt, va_list ap)
+void log_do_log_daemon(int prio, void *priv, const char *fmt, va_list ap)
 {
-	if (log_daemon) {
-		la->ops[0].sem_op = -1;
-		if (semop(la->semid, la->ops, 1) < 0) {
-			syslog(LOG_ERR, "semop up failed");
-			return;
-		}
+	struct sembuf ops[1];
 
-		log_enqueue(prio, fmt, ap);
+	ops[0].sem_num = la->ops[0].sem_num;
+	ops[0].sem_flg = la->ops[0].sem_flg;
 
-		la->ops[0].sem_op = 1;
-		if (semop(la->semid, la->ops, 1) < 0) {
-			syslog(LOG_ERR, "semop down failed");
-			return;
-		}
+	ops[0].sem_op = -1;
+	if (semop(la->semid, ops, 1) < 0) {
+		syslog(LOG_ERR, "semop down failed %d", errno);
+		return;
+	}
+
+	log_enqueue(prio, fmt, ap);
+
+	ops[0].sem_op = 1;
+	if (semop(la->semid, ops, 1) < 0)
+		syslog(LOG_ERR, "semop up failed");
+}
+
+void log_do_log_std(int prio, void *priv, const char *fmt, va_list ap)
+{
+	if (prio == LOG_INFO) {
+		vfprintf(stdout, fmt, ap);
+		fprintf(stdout, "\n");
 	} else {
 		fprintf(stderr, "%s: ", log_name);
 		vfprintf(stderr, fmt, ap);
@@ -256,7 +296,7 @@ void log_warning(const char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap, fmt);
-	dolog(LOG_WARNING, fmt, ap);
+	log_func(LOG_WARNING, log_func_priv, fmt, ap);
 	va_end(ap);
 }
 
@@ -264,7 +304,7 @@ void log_error(const char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap, fmt);
-	dolog(LOG_ERR, fmt, ap);
+	log_func(LOG_ERR, log_func_priv, fmt, ap);
 	va_end(ap);
 }
 
@@ -273,11 +313,19 @@ void log_debug(int level, const char *fmt, ...)
 	if (log_level > level) {
 		va_list ap;
 		va_start(ap, fmt);
-		dolog(LOG_DEBUG, fmt, ap);
+		log_func(LOG_DEBUG, log_func_priv, fmt, ap);
 		va_end(ap);
 	}
 }
 
+void log_info(const char *fmt, ...)
+{
+	va_list ap;
+	va_start(ap, fmt);
+	log_func(LOG_INFO, log_func_priv, fmt, ap);
+	va_end(ap);
+}
+
 static void __dump_line(int level, unsigned char *buf, int *cp)
 {
 	char line[16*3+5], *lp = line;
@@ -314,19 +362,27 @@ static void __dump_char(int level, unsigned char *buf, int *cp, int ch)
 
 static void log_flush(void)
 {
+	int msglen;
+	struct sembuf ops[1];
+
+	ops[0].sem_num = la->ops[0].sem_num;
+	ops[0].sem_flg = la->ops[0].sem_flg;
+
+
 	while (!la->empty) {
-		la->ops[0].sem_op = -1;
-		if (semop(la->semid, la->ops, 1) < 0) {
+		ops[0].sem_op = -1;
+		if (semop(la->semid, ops, 1) < 0) {
+			syslog(LOG_ERR, "semop down failed %d", errno);
+			exit(1);
+		}
+		msglen = log_dequeue(la->buff);
+		ops[0].sem_op = 1;
+		if (semop(la->semid, ops, 1) < 0) {
 			syslog(LOG_ERR, "semop up failed");
 			exit(1);
 		}
-		log_dequeue(la->buff);
-		la->ops[0].sem_op = 1;
-		if (semop(la->semid, la->ops, 1) < 0) {
-			syslog(LOG_ERR, "semop down failed");
-			exit(1);
-		}
-		log_syslog(la->buff);
+		if (msglen)
+			log_syslog(la->buff);
 	}
 }
 
@@ -346,20 +402,23 @@ static void catch_signal(int signo)
 
 static void __log_close(void)
 {
-	if (log_daemon) {
-		fprintf(stderr, "close log\n");
+	if (log_func == log_do_log_daemon) {
 		log_flush();
 		closelog();
 		free_logarea();
 	}
 }
 
-int log_init(char *program_name, int size)
+int log_init(char *program_name, int size,
+	void (*func)(int prio, void *priv, const char *fmt, va_list ap),
+	void *priv)
 {
 	logdbg(stderr,"enter log_init\n");
 	log_name = program_name;
+	log_func = func;
+	log_func_priv = priv;
 
-	if (log_daemon) {
+	if (log_func == log_do_log_daemon) {
 		struct sigaction sa_old;
 		struct sigaction sa_new;
 		pid_t pid;
@@ -367,8 +426,10 @@ int log_init(char *program_name, int size)
 		openlog(log_name, 0, LOG_DAEMON);
 		setlogmask (LOG_UPTO (LOG_DEBUG));
 
-		if (logarea_init(size))
+		if (logarea_init(size)) {
+			syslog(LOG_ERR, "logarea init failed");
 			return -1;
+		}
 
 		pid = fork();
 		if (pid < 0) {
@@ -408,7 +469,7 @@ void log_close(pid_t pid)
 {
 	int status;
 
-	if (!log_daemon || pid < 0) {
+	if (log_func != log_do_log_daemon || pid < 0) {
 		__log_close();
 		return;
 	}
