@@ -188,7 +188,6 @@ iscsi_update_address(iscsi_conn_t *conn, char *address)
 	char *port, *tag;
 	char default_port[NI_MAXSERV];
 	iscsi_session_t *session = conn->session;
-
 	struct sockaddr_storage addr;
 
 	if ((tag = strrchr(address, ','))) {
@@ -205,6 +204,18 @@ iscsi_update_address(iscsi_conn_t *conn, char *address)
 		port = default_port;
 	}
 
+	if (*address == '[') {
+		char *end_bracket;
+
+		if (!(end_bracket = strchr(address, ']'))) {
+			log_error("Invalid IPv6 address with opening bracket, "
+				  "but no closing bracket.");
+			return 0;
+		}
+		*end_bracket = '\0';
+		address++;
+        }
+
 	if (resolve_address(address, port, &addr)) {
 		log_error("cannot resolve host name %s", address);
 		return 0;
@@ -264,24 +275,20 @@ get_security_text_keys(iscsi_session_t *session, int cid, char **data,
 					 &value, &value_end)) {
 		/*
 		 * We should have already obtained this
-		 * via discovery.
-		 * We've already picked an isid, so the
-		 * most we can do is confirm we reached
-		 * the portal group we were expecting to
+		 * via discovery, but the value could be stale.
+		 * If the target was reconfigured it will send us
+		 * the updated tpgt.
 		 */
 		tag = strtoul(value, NULL, 0);
 		if (session->portal_group_tag >= 0) {
-			if (tag != session->portal_group_tag) {
-				log_error("Portal group tag "
+			if (tag != session->portal_group_tag)
+				log_debug(2, "Portal group tag "
 					  "mismatch, expected %u, "
-					  "received %u",
+					  "received %u. Updating",
 					  session->portal_group_tag, tag);
-				return LOGIN_WRONG_PORTAL_GROUP;
-			}
-		} else
-			/* we now know the tag */
-			session->portal_group_tag = tag;
-
+		}
+		/* we now know the tag */
+		session->portal_group_tag = tag;
 		text = value_end;
 	} else {
 		/*
@@ -339,21 +346,22 @@ get_op_params_text_keys(iscsi_session_t *session, int cid,
 		}
 	} else if (iscsi_find_key_value("TargetPortalGroupTag", text, end,
 					 &value, &value_end)) {
-		/*
-		 * confirm we reached the portal group we were expecting to
-		 */
 		int tag = strtoul(value, NULL, 0);
+		/*
+		 * We should have already obtained this
+		 * via discovery, but the value could be stale.
+		 * If the target was reconfigured it will send us
+		 * the updated tpgt.
+		 */
 		if (session->portal_group_tag >= 0) {
-			if (tag != session->portal_group_tag) {
-				log_error("Portal group tag mismatch, "
-					  "expected %u, received %u",
+			if (tag != session->portal_group_tag)
+				log_debug(2, "Portal group tag "
+					  "mismatch, expected %u, "
+					  "received %u. Updating",
 					  session->portal_group_tag, tag);
-				return LOGIN_WRONG_PORTAL_GROUP;
-			}
-		} else
-			/* we now know the tag */
-			session->portal_group_tag = tag;
-
+		}
+		/* we now know the tag */
+		session->portal_group_tag = tag;
 		text = value_end;
 	} else if (iscsi_find_key_value("InitialR2T", text, end, &value,
 					 &value_end)) {
@@ -380,8 +388,19 @@ get_op_params_text_keys(iscsi_session_t *session, int cid,
 	} else if (iscsi_find_key_value("MaxRecvDataSegmentLength", text, end,
 				     &value, &value_end)) {
 		if (session->type == ISCSI_SESSION_TYPE_DISCOVERY ||
-		    !session->t->template->rdma)
-			conn->max_xmit_dlength = strtoul(value, NULL, 0);
+		    !session->t->template->rdma) {
+			int tgt_max_xmit;
+			conn_rec_t *conn_rec = &session->nrec.conn[cid];
+
+			tgt_max_xmit = strtoul(value, NULL, 0);
+			/*
+			 * if the rec value is zero it means to use
+			 * what the target gave us.
+			 */
+			if (!conn_rec->iscsi.MaxXmitDataSegmentLength ||
+			    tgt_max_xmit < conn->max_xmit_dlength)
+				conn->max_xmit_dlength = tgt_max_xmit;
+		}
 		text = value_end;
 	} else if (iscsi_find_key_value("FirstBurstLength", text, end, &value,
 					 &value_end)) {
@@ -1399,8 +1418,6 @@ iscsi_login_req(iscsi_session_t *session, iscsi_login_context_t *c)
 		c->ret = LOGIN_IO_ERROR;
 		goto done;
 	}
-
-	conn->state = STATE_IN_LOGIN;
 	return 0;
 
  done:
@@ -1464,8 +1481,6 @@ iscsi_login_rsp(iscsi_session_t *session, iscsi_login_context_t *c)
 		     &c->final);
 	if (c->final)
 		goto done;
-
-	conn->state = STATE_FREE;
 	return 0;
 
  done:
@@ -1501,7 +1516,6 @@ iscsi_login(iscsi_session_t *session, int cid, char *buffer, size_t bufsize,
 	iscsi_login_context_t *c = &conn->login_context;
 	int ret;
 
-	conn->kernel_io = 0;
 	/*
 	 * assume iscsi_login is only called from discovery, so it is
 	 * safe to always set to zero
