@@ -47,6 +47,7 @@ typedef enum mgmt_ipc_err {
 	MGMT_IPC_ERR_ISNS_UNAVAILABLE	= 17,
 	MGMT_IPC_ERR_ISCSID_COMM_ERR	= 18,
 	MGMT_IPC_ERR_FATAL_LOGIN_FAILURE = 19,
+	MGMT_IPC_ERR_ISCSID_NOTCONN	= 20,
 } mgmt_ipc_err_e;
 
 typedef enum iscsiadm_cmd {
@@ -133,14 +134,10 @@ typedef struct iscsiadm_rsp {
 struct queue_task;
 typedef mgmt_ipc_err_e	mgmt_ipc_fn_t(struct queue_task *);
 
-struct iscsi_ipc *ipc;
-
-void need_reap(void);
-void event_loop(struct iscsi_ipc *ipc, int control_fd, int mgmt_ipc_fd, int isns_fd);
-
 struct queue_task;
 void mgmt_ipc_write_rsp(struct queue_task *qtask, mgmt_ipc_err_e err);
 int mgmt_ipc_listen(void);
 void mgmt_ipc_close(int fd);
+void mgmt_ipc_handle(int accept_fd);
 
 #endif /* MGMT_IPC_H */
